(:~
 : XQuery portion of "Example of XQuery/JavaScript Combination" application.
 : This file belongs in the folder to which org.basex.restxqpath points.
 : Access point for application: /static/xquery_js_example.html
 :
 : Copyright 2016 The MathWorks, Inc.
 :)

module namespace xquery_js_example = 'http://mathworks.com/xquery/restxq';

declare variable $xquery_js_example:myxml := 
   <chapter>
      <title>Sample Content (realistic application would use XML files or XQuery database)</title>
      <section>
        <title>Section 1</title>
        <para>First para of section 1</para>
      </section>
      <section>
        <title>Section 2</title>
        <para>First para of section 2</para>
        <para>Second para of section 2</para>
      </section>
   </chapter>;

(:~
 : Count elements and pass results back to JavaScript.
 : @param element Name of XML element to count
 :
 : @return <script> element
 :)
declare
   %rest:path("xquery_js_example/{$element}")
   %output:method("xhtml")
   %output:omit-xml-declaration("yes")  
   function xquery_js_example:myfcn(
      $element as xs:string
   ) as element() {  
   let $count := count($xquery_js_example:myxml/descendant-or-self::*[local-name(.)=$element])     
   return <script>{'var itemArray = ' || xquery_js_example:serialize_json($count) || ';'}</script>
};

(:~
 : Create JSON array with designated number of items.
 : @param count Number of items in array
 :
 : @return Serialized JSON array
 :)
declare function xquery_js_example:serialize_json(
   $count as xs:integer
) as xs:string {
   serialize(  
      <json type="array">{
         for $this in (1 to $count)
         return
         <_ type="object">
            <itemID>{$this}</itemID>
         </_>
      }</json>,
      <output:serialization-parameters xmlns:output="http://www.w3.org/2010/xslt-xquery-serialization">
         <output:method value="json"/>
      </output:serialization-parameters>
   )  
};

(:~ 
 : Return XML document serialized as text
 :)
declare %rest:path("xquery_js_example/get_doc")
  %output:method("text")
  %output:omit-xml-declaration("yes")
  %output:indent("yes") 
  function xquery_js_example:get-doc() as xs:string {
    serialize($xquery_js_example:myxml)
};
